@@ -1,11 +1,45 @@
+from functools import partial
 from typing import Tuple
 
 import jax
 import jax.numpy as jnp
-import numpy as np
 from flax import struct
-from jax import jit
-from jax.experimental.host_callback import call
+
+init_dice_pattern: jnp.ndarray = jnp.array(
+    [
+        [0, 1],
+        [0, 2],
+        [0, 3],
+        [0, 4],
+        [0, 5],
+        [1, 0],
+        [1, 2],
+        [1, 3],
+        [1, 4],
+        [1, 5],
+        [2, 0],
+        [2, 1],
+        [2, 3],
+        [2, 4],
+        [2, 5],
+        [3, 0],
+        [3, 1],
+        [3, 2],
+        [3, 4],
+        [3, 5],
+        [4, 0],
+        [4, 1],
+        [4, 2],
+        [4, 3],
+        [4, 5],
+        [5, 0],
+        [5, 1],
+        [5, 2],
+        [5, 3],
+        [5, 4],
+    ],
+    dtype=jnp.int16,
+)
 
 
 @struct.dataclass
@@ -38,7 +72,6 @@ class BackgammonState:
     legal_action_mask: jnp.ndarray = jnp.zeros(6 * 26 + 6, dtype=jnp.int16)
 
 
-@jit
 def init(rng: jax.random.KeyArray) -> Tuple[jnp.ndarray, BackgammonState]:
     rng1, rng2, rng3 = jax.random.split(rng, num=3)
     curr_player: jnp.ndarray = jnp.int16(jax.random.bernoulli(rng1))
@@ -65,18 +98,34 @@ def init(rng: jax.random.KeyArray) -> Tuple[jnp.ndarray, BackgammonState]:
     return curr_player, state
 
 
-@jit
 def step(
     state: BackgammonState, action: int
-) -> Tuple[BackgammonState, int, bool]:
+) -> Tuple[jnp.ndarray, BackgammonState, int]:
+    """
+    step 関数.
+    terminatedしている場合, 状態をそのまま返す.
+    """
     return jax.lax.cond(
-        _is_turn_end(state),
-        lambda: (_change_turn(state).curr_player, _change_turn(state), 0),
+        state.terminated,
+        lambda: (state.curr_player, state, 0),
         lambda: _normal_step(state, action),
     )
 
 
-@jit
+def _normal_step(
+    state: BackgammonState, action: int
+) -> Tuple[jnp.ndarray, BackgammonState, int]:
+    """
+    terminated していない場合のstep 関数.
+    """
+    state = _update_by_action(state, action)
+    return jax.lax.cond(
+        _is_all_off(state.board, state.turn),
+        lambda: _winning_step(state),
+        lambda: _no_winning_step(state),
+    )
+
+
 def observe(state: BackgammonState, curr_player: jnp.ndarray) -> jnp.ndarray:
     """
     手番のplayerに対する観測を返す.
@@ -94,37 +143,31 @@ def observe(state: BackgammonState, curr_player: jnp.ndarray) -> jnp.ndarray:
     )
 
 
-@jit
 def _to_zero_one_dice_vec(playable_dice: jnp.ndarray) -> jnp.ndarray:
     """
     playできるサイコロを6次元の0-1ベクトルで返す.
     """
-    zero_one_dice_vec: jnp.ndarray = jnp.zeros(6, dtype=jnp.int16)
-    return jax.lax.fori_loop(
-        0,
-        4,
-        lambda i, x: jax.lax.cond(
-            playable_dice[i] != -1,
-            lambda: x.at[playable_dice[i]].set(1),
-            lambda: x,
-        ),
-        zero_one_dice_vec,
+    dice_indices: jnp.ndarray = jnp.array(
+        [0, 1, 2, 3], dtype=jnp.int16
+    )  # サイコロの数は最大4
+
+    def _insert_dice_num(
+        idx: jnp.ndarray, playable_dice: jnp.ndarray
+    ) -> jnp.ndarray:
+        vec: jnp.ndarray = jnp.zeros(6, dtype=jnp.int16)
+        return (playable_dice[idx] != -1) * vec.at[playable_dice[idx]].set(
+            1
+        ) + (playable_dice[idx] == -1) * vec
+
+    return (
+        jax.vmap(_insert_dice_num)(
+            dice_indices, jnp.tile(playable_dice, (4, 1))
+        )
+        .sum(axis=0)
+        .astype(jnp.int16)
     )
 
 
-@jit
-def _normal_step(
-    state: BackgammonState, action: int
-) -> Tuple[BackgammonState, int, bool]:
-    state = _update_by_action(state, action)
-    return jax.lax.cond(
-        _is_all_off(state.board, state.turn),
-        lambda: _winning_step(state),
-        lambda: _no_winning_step(state),
-    )
-
-
-@jit
 def _winning_step(
     state: BackgammonState,
 ) -> Tuple[jnp.ndarray, BackgammonState, int]:
@@ -136,25 +179,31 @@ def _winning_step(
     return state.curr_player, state, reward
 
 
-@jit
 def _no_winning_step(
     state: BackgammonState,
 ) -> Tuple[jnp.ndarray, BackgammonState, int]:
     """
     勝利者がいない場合のstep, ターン終了の条件を満たせばターンを変更する.
     """
+    s = _change_until_legal(state)
     return jax.lax.cond(
         _is_turn_end(state),
         lambda: (
-            _change_turn(state).curr_player,
-            _change_turn(state),
+            s.curr_player,
+            s,
             0,
         ),
         lambda: (state.curr_player, state, 0),
     )
 
 
-@jit
+def _change_until_legal(state: BackgammonState) -> BackgammonState:
+    """
+    行動可能なplayerが出るまでturnを変え続ける.
+    """
+    return jax.lax.while_loop(_is_turn_end, _change_turn, state)
+
+
 def _update_by_action(state: BackgammonState, action: int) -> BackgammonState:
     """
     行動を受け取って状態をupdate
@@ -183,21 +232,43 @@ def _update_by_action(state: BackgammonState, action: int) -> BackgammonState:
     )
 
 
-@jit
 def _make_init_board() -> jnp.ndarray:
-    board: jnp.ndarray = jnp.zeros(28, dtype=jnp.int16)
-    board = board.at[0].set(-2)
-    board = board.at[5].set(5)
-    board = board.at[7].set(3)
-    board = board.at[11].set(-5)
-    board = board.at[12].set(5)
-    board = board.at[16].set(-3)
-    board = board.at[18].set(-5)
-    board = board.at[23].set(2)
+    board: jnp.ndarray = jnp.array(
+        [
+            -2,
+            0,
+            0,
+            0,
+            0,
+            5,
+            0,
+            3,
+            0,
+            0,
+            0,
+            -5,
+            5,
+            0,
+            0,
+            0,
+            -3,
+            0,
+            -5,
+            0,
+            0,
+            0,
+            0,
+            2,
+            0,
+            0,
+            0,
+            0,
+        ],
+        dtype=jnp.int16,
+    )
     return board
 
 
-@jit
 def _is_turn_end(state: BackgammonState) -> bool:
     """
     play可能なサイコロ数が0の場合ないしlegal_actionがない場合交代
@@ -207,8 +278,7 @@ def _is_turn_end(state: BackgammonState) -> bool:
     )  # type: ignore
 
 
-@jit
-def _change_turn(state: BackgammonState) -> Tuple[BackgammonState]:
+def _change_turn(state: BackgammonState) -> BackgammonState:
     """
     ターンを変更して新しい状態を返す.
     """
@@ -234,27 +304,14 @@ def _change_turn(state: BackgammonState) -> Tuple[BackgammonState]:
     )
 
 
-@jax.jit
 def _roll_init_dice(rng: jax.random.KeyArray) -> jnp.ndarray:
     """
     # 違う目が出るまで振り続ける.
     """
 
-    def _cond_fn(roll: jnp.ndarray):
-        return roll[0] == roll[1]
-
-    def _body_fn(_roll: jnp.ndarray):
-        roll: jnp.ndarray = jax.random.randint(
-            rng, shape=(1, 2), minval=0, maxval=6, dtype=jnp.int16
-        )
-        return roll[0]
-
-    return jax.lax.while_loop(
-        _cond_fn, _body_fn, jnp.array([0, 0], dtype=jnp.int16)
-    )
+    return jax.random.choice(rng, init_dice_pattern)
 
 
-@jit
 def _roll_dice(rng: jax.random.KeyArray) -> jnp.ndarray:
     roll: jnp.ndarray = jax.random.randint(
         rng, shape=(1, 2), minval=0, maxval=6, dtype=jnp.int16
@@ -262,29 +319,24 @@ def _roll_dice(rng: jax.random.KeyArray) -> jnp.ndarray:
     return roll[0]
 
 
-@jit
 def _init_turn(dice: jnp.ndarray) -> jnp.ndarray:
     """
     ゲーム開始時のターン決め.
     サイコロの目が大きい方が手番.
     """
     diff = dice[1] - dice[0]
-    return jax.lax.cond(diff > 0, lambda: jnp.int16(1), lambda: jnp.int16(-1))
+    return (diff > 0) * jnp.int16(1) + (diff <= 0) * jnp.int16(-1)
 
 
-@jit
 def _set_playable_dice(dice: jnp.ndarray) -> jnp.ndarray:
     """
     -1でemptyを表す.
     """
-    return jax.lax.cond(
-        dice[0] == dice[1],
-        lambda: jnp.array([dice[0]] * 4, dtype=np.int16),
-        lambda: jnp.array([dice[0], dice[1], -1, -1], dtype=np.int16),
-    )
+    return (dice[0] == dice[1]) * jnp.array([dice[0]] * 4, dtype=jnp.int16) + (
+        dice[0] != dice[1]
+    ) * jnp.array([dice[0], dice[1], -1, -1], dtype=jnp.int16)
 
 
-@jit
 def _update_playable_dice(
     playable_dice: jnp.ndarray,
     played_dice_num: jnp.ndarray,
@@ -292,75 +344,64 @@ def _update_playable_dice(
     action: int,
 ) -> jnp.ndarray:
     _n = played_dice_num
-    die = action % 6
+    die_array = jnp.array([action % 6] * 4, dtype=jnp.int16)
+    dice_indices: jnp.ndarray = jnp.array(
+        [0, 1, 2, 3], dtype=jnp.int16
+    )  # サイコロの数は最大4
 
-    @jit
-    def _update_for_diff_dice(die: int, playable_dice: np.ndarray):
-        return jax.lax.fori_loop(
-            0,
-            4,
-            lambda i, x: jax.lax.cond(
-                die == x[i], lambda: x.at[i].set(-1), lambda: x
-            ),
-            playable_dice,
-        )
+    def _update_for_diff_dice(
+        die: jnp.ndarray, idx: jnp.ndarray, playable_dice: jnp.ndarray
+    ):
+        return (die == playable_dice[idx]) * -1 + (
+            die != playable_dice[idx]
+        ) * playable_dice[idx]
 
-    return jax.lax.cond(
-        dice[0] == dice[1],
-        lambda: playable_dice.at[3 - _n].set(-1),
-        lambda: _update_for_diff_dice(die, playable_dice),
+    return (dice[0] == dice[1]) * playable_dice.at[3 - _n].set(-1) + (
+        dice[0] != dice[1]
+    ) * jax.vmap(_update_for_diff_dice)(
+        die_array, dice_indices, jnp.tile(playable_dice, (4, 1))
+    ).astype(
+        jnp.int16
     )
 
 
-@jit
 def _home_board(turn: jnp.ndarray) -> jnp.ndarray:
     """
     黒: [18~23], 白: [0~5]
     """
-    return jax.lax.cond(
-        turn == -1, lambda: jnp.arange(18, 24), lambda: jnp.arange(0, 6)
-    )
+    return (turn == -1) * jnp.arange(18, 24) + (turn == 1) * jnp.arange(0, 6)  # type: ignore
 
 
-@jit
 def _off_idx(turn: jnp.ndarray) -> int:
     """
     黒: 26, 白: 27
     """
-    return jax.lax.cond(turn == -1, lambda: 26, lambda: 27)
+    return (turn == -1) * 26 + (turn == 1) * 27  # type: ignore
 
 
-@jit
 def _bar_idx(turn: jnp.ndarray) -> int:
     """
     黒: 24, 白 25
     """
-    return jax.lax.cond(turn == -1, lambda: 24, lambda: 25)
+    return (turn == -1) * 24 + (turn == 1) * 25  # type: ignore
 
 
-@jit
 def _rear_distance(board: jnp.ndarray, turn: jnp.ndarray) -> jnp.ndarray:
     """
     board上にあるcheckerについて, goal地点とcheckerの距離の最大値
     """
     b = board[:24]
-
     exists = jnp.where((b * turn > 0), size=24, fill_value=jnp.nan)[  # type: ignore
         0
     ]
-    return jax.lax.cond(
-        turn == 1,
-        lambda: jnp.int16(
-            jnp.max(jnp.nan_to_num(exists, nan=jnp.int16(-100))) + 1
-        ),
-        lambda: jnp.int16(
-            24 - jnp.min(jnp.nan_to_num(exists, nan=jnp.int16(100)))
-        ),
+    return (turn == 1) * jnp.int16(
+        jnp.max(jnp.nan_to_num(exists, nan=jnp.int16(-100))) + 1
+    ) + (turn == -1) * jnp.int16(
+        24 - jnp.min(jnp.nan_to_num(exists, nan=jnp.int16(100)))
     )
 
 
-@jit
-def _is_all_on_homeboad(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
+def _is_all_on_home_board(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
     """
     全てのcheckerがhome boardにあれば, bear offできる.
     """
@@ -372,7 +413,6 @@ def _is_all_on_homeboad(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
     return (15 - off) == on_home_board
 
 
-@jit
 def _is_open(board: jnp.ndarray, turn: jnp.ndarray, point: int) -> bool:
     """
     手番のplayerにとって, pointが空いてるかを判定する.
@@ -382,7 +422,6 @@ def _is_open(board: jnp.ndarray, turn: jnp.ndarray, point: int) -> bool:
     return turn * checkers >= -1  # type: ignore
 
 
-@jit
 def _exists(board: jnp.ndarray, turn: jnp.ndarray, point: int) -> bool:
     """
     指定pointに手番のchckerが存在するか.
@@ -391,40 +430,37 @@ def _exists(board: jnp.ndarray, turn: jnp.ndarray, point: int) -> bool:
     return turn * checkers >= 1  # type: ignore
 
 
-@jit
-def _calc_src(src: int, turn: jnp.ndarray) -> jnp.ndarray:
+def _calc_src(src: int, turn: jnp.ndarray) -> int:
     """
     boardのindexに合わせる.
     """
-    return jax.lax.cond(
-        src == 1, lambda: jnp.int16(_bar_idx(turn)), lambda: jnp.int16(src - 2)
-    )
+    return (src == 1) * jnp.int16(_bar_idx(turn)) + (src != 1) * jnp.int16(
+        src - 2
+    )  # type: ignore
 
 
-@jit
-def _calc_tgt(src: int, turn: jnp.ndarray, die) -> jnp.ndarray:
+def _calc_tgt(src: int, turn: jnp.ndarray, die) -> int:
     """
-    boardのindexに合わせる.
+    boardのindexに合わせる. actionは src*6 + dieの形になっている. targetは黒ならsrcからdie分+白ならdie分-(目的地が逆だから.)
     """
-    return jax.lax.cond(
-        src >= 24,
-        lambda: jnp.int16(
-            jnp.clip(24 * turn, a_min=-1, a_max=24) + die * -1 * turn
-        ),
-        lambda: jnp.int16(_from_other_than_bar(src, turn, die)),
+    return (src >= 24) * jnp.int16(
+        jnp.clip(24 * turn, a_min=-1, a_max=24) + die * -1 * turn
+    ) + (src < 24) * jnp.int16(
+        _from_other_than_bar(src, turn, die)
     )  # type: ignore
 
 
-@jit
 def _from_other_than_bar(src: int, turn: jnp.ndarray, die: int) -> int:
-    return jax.lax.cond(
-        (src + die * -1 * turn >= 0) & (src + die * -1 * turn <= 23),
-        lambda: jnp.int16(src + die * -1 * turn),
-        lambda: jnp.int16(_off_idx(turn)),
+    _is_from_board = (src + die * -1 * turn >= 0) & (
+        src + die * -1 * turn <= 23
+    )
+    return _is_from_board * jnp.int16(src + die * -1 * turn) + (
+        (~_is_from_board)
+    ) * jnp.int16(
+        _off_idx(turn)
     )  # type: ignore
 
 
-@jit
 def _decompose_action(action: int, turn: jnp.ndarray) -> Tuple:
     """
     action(int)をsource, die, tagetに分解する.
@@ -435,12 +471,6 @@ def _decompose_action(action: int, turn: jnp.ndarray) -> Tuple:
     return src, die, tgt
 
 
-@jit
-def selu(x, alpha=1.67, lmbda=1.05):
-    call(lambda x: print(f"x: {x}"), x)
-
-
-@jit
 def _is_action_legal(board: jnp.ndarray, turn, action: int) -> bool:
     """
     micro actionの合法判定
@@ -448,52 +478,48 @@ def _is_action_legal(board: jnp.ndarray, turn, action: int) -> bool:
     src = [no op., from bar, 0, .., 23]
     """
     src, die, tgt = _decompose_action(action, turn)
-    return jax.lax.cond(
-        (0 <= tgt) & (tgt <= 23) & (src >= 0),
-        lambda: _is_to_point_legal(board, turn, src, tgt),
-        lambda: _is_to_off_legal(board, turn, src, tgt, die),
-    )
+    _is_to_point = (0 <= tgt) & (tgt <= 23) & (src >= 0)
+    return _is_to_point & _is_to_point_legal(board, turn, src, tgt) | (
+        ~_is_to_point
+    ) & _is_to_off_legal(
+        board, turn, src, tgt, die
+    )  # type: ignore
 
 
-@jit
 def _distance_to_goal(src: int, turn: jnp.ndarray) -> int:
-    return jax.lax.cond(turn == -1, lambda: 24 - src, lambda: src + 1)  # type: ignore
+    return (turn == -1) * (24 - src) + (turn == 1) * (src + 1)  # type: ignore
 
 
-@jit
 def _is_to_off_legal(
     board: jnp.ndarray, turn: jnp.ndarray, src: int, tgt: int, die: int
-) -> bool:
+):
     """
     board外への移動についての合法判定
     """
-    return jax.lax.cond(
-        src < 0,
-        lambda: False,
-        lambda: _exists(board, turn, src)
-        & _is_all_on_homeboad(board, turn)
-        & (_rear_distance(board, turn) <= die)
-        & (_rear_distance(board, turn) == _distance_to_goal(src, turn)),
+    r = _rear_distance(board, turn)
+    d = _distance_to_goal(src, turn)
+    return (
+        (src >= 0)
+        & _exists(board, turn, src)
+        & _is_all_on_home_board(board, turn)
+        & (r <= die)
+        & (r == d)
     )  # type: ignore
 
 
-@jit
 def _is_to_point_legal(
     board: jnp.ndarray, turn: jnp.ndarray, src: int, tgt: int
 ) -> bool:
     """
     tgtがpointの場合の合法手判定
     """
-    return jax.lax.cond(
-        src >= 24,
-        lambda: (_exists(board, turn, src)) & (_is_open(board, turn, tgt)),
-        lambda: (_exists(board, turn, src))
-        & (_is_open(board, turn, tgt))
-        & (board[_bar_idx(turn)] == 0),
-    )
+    e = _exists(board, turn, src)
+    o = _is_open(board, turn, tgt)
+    return ((src >= 24) & e & o) | (
+        (src < 24) & e & o & (board[_bar_idx(turn)] == 0)
+    )  # type: ignore
 
 
-@jit
 def _move(board: jnp.ndarray, turn: jnp.ndarray, action: int) -> jnp.ndarray:
     """
     micro actionに基づく状態更新
@@ -509,7 +535,6 @@ def _move(board: jnp.ndarray, turn: jnp.ndarray, action: int) -> jnp.ndarray:
     return board
 
 
-@jit
 def _is_all_off(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
     """
     手番のプレイヤーのチェッカーが全てoffにあれば勝利となる.
@@ -517,25 +542,16 @@ def _is_all_off(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
     return board[_off_idx(turn)] * turn == 15  # type: ignore
 
 
-@jit
 def _calc_win_score(board: jnp.ndarray, turn: jnp.ndarray) -> int:
-    return jax.lax.cond(
-        _is_gammon(board, turn),
-        lambda: _score(board, turn),
-        lambda: 1,
-    )
+    """
+    通常勝ち: 1点
+    gammon勝ち: 2点
+    backgammon勝ち: 3点
+    """
+    g = _is_gammon(board, turn)
+    return 1 + g + (g & _remains_at_inner(board, turn))
 
 
-@jit
-def _score(board: jnp.ndarray, turn: jnp.ndarray) -> int:
-    return jax.lax.cond(
-        _remains_at_inner(board, turn),
-        lambda: 3,
-        lambda: 2,
-    )
-
-
-@jit
 def _is_gammon(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
     """
     相手のoffに一つもcheckerがなければgammon勝ち
@@ -543,7 +559,6 @@ def _is_gammon(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
     return board[_off_idx(-1 * turn)] == 0  # type: ignore
 
 
-@jit
 def _remains_at_inner(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
     """
     相手のoffに一つもcheckerがない && 相手のcheckerが一つでも自分のインナーに残っている
@@ -552,51 +567,48 @@ def _remains_at_inner(board: jnp.ndarray, turn: jnp.ndarray) -> bool:
     return jnp.take(board, _home_board(-1 * turn)).sum() != 0  # type: ignore
 
 
-@jit
 def _legal_action_mask(
     board: jnp.ndarray, turn: jnp.ndarray, dice: jnp.ndarray
 ) -> jnp.ndarray:
-    legal_action_mask = jnp.zeros(26 * 6 + 6, dtype=np.int16)
 
-    @jit
-    def _update(i: int, legal_action_mask: jnp.ndarray) -> jnp.ndarray:
-        return legal_action_mask | _legal_action_mask_for_single_die(
-            board, turn, dice[i]
-        )
-
-    legal_action_mask = jax.lax.fori_loop(0, 4, _update, legal_action_mask)
+    legal_action_mask = jax.vmap(
+        partial(_legal_action_mask_for_single_die, board=board, turn=turn)
+    )(die=dice).any(
+        axis=0
+    )  # (26*6 + 6)
     return legal_action_mask
 
 
-@jit
 def _legal_action_mask_for_single_die(
-    board: jnp.ndarray, turn: jnp.ndarray, die: int
+    board: jnp.ndarray, turn: jnp.ndarray, die
 ) -> jnp.ndarray:
     """
     一つのサイコロの目に対するlegal micro action
     """
-    return jax.lax.cond(
-        die == -1,
-        lambda: jnp.zeros(26 * 6 + 6, dtype=np.int16),
-        lambda: _legal_action_mask_for_valid_single_dice(board, turn, die),
-    )
+    return (die == -1) * jnp.zeros(26 * 6 + 6, dtype=jnp.int16) + (
+        die != -1
+    ) * _legal_action_mask_for_valid_single_dice(board, turn, die)
 
 
-@jit
 def _legal_action_mask_for_valid_single_dice(
-    board: jnp.ndarray, turn: jnp.ndarray, die: int
+    board: jnp.ndarray, turn: jnp.ndarray, die
 ) -> jnp.ndarray:
     """
     -1以外のサイコロの目に対して合法判定
     """
-    legal_action_mask = jnp.zeros(26 * 6 + 6, dtype=np.int16)
+    src_indices = jnp.arange(
+        26, dtype=jnp.int16
+    )  # 26パターンのsrcに対してlegal_actionを求める.
 
-    @jit
-    def _is_legal(i: int, legal_action_mask: jnp.ndarray):
-        action = i * 6 + die
+    def _is_legal(idx: jnp.ndarray):
+        action: int = idx * 6 + die
+        legal_action_mask = jnp.zeros(26 * 6 + 6, dtype=jnp.int16)
         legal_action_mask = legal_action_mask.at[action].set(
             _is_action_legal(board, turn, action)
         )
         return legal_action_mask
 
-    return jax.lax.fori_loop(0, 26, _is_legal, legal_action_mask)
+    legal_action_mask = jax.vmap(_is_legal)(src_indices).any(
+        axis=0
+    )  # (26*6 + 6)
+    return legal_action_mask
