@@ -7,7 +7,7 @@ from flax.struct import dataclass
 
 EnvId = Literal[
     "tic_tac_toe/v0",
-    "minatar/breakout/v0",
+    "shogi/v0" "minatar/breakout/v0",
     "suzume_jong/v0",
     "go/v0",
 ]
